unit Main;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  System.UIConsts, System.StrUtils,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.TMSFNCTypes, FMX.TMSFNCUtils, FMX.TMSFNCGraphics, FMX.TMSFNCGraphicsTypes,
  FMX.StdCtrls, FMX.Colors, FMX.Objects, FMX.TMSFNCScrollControl,
  FMX.TMSFNCRichEditorBase, FMX.TMSFNCRichEditor, FMX.Controls.Presentation,
  FMX.TMSFNCButton, FMX.TMSFNCStatusBar, FMX.TMSFNCCustomControl,
  FMX.TMSFNCPanel, FMX.TMSFNCListBox, FMX.Memo.Types, FMX.ScrollBox, FMX.Memo,
  FMX.Ani;

type
  TfrmMain = class(TForm)
    fncPanel: TTMSFNCPanel;
    fncStatusBar: TTMSFNCStatusBar;
    fncButtonOpen: TTMSFNCButton;
    fncRichEditor: TTMSFNCRichEditor;
    fncButtonSave: TTMSFNCButton;
    fncButtonFirst: TTMSFNCButton;
    fncButtonNext: TTMSFNCButton;
    fncButtonApply: TTMSFNCButton;
    fncButtonApplyAll: TTMSFNCButton;
    dlgOpen: TOpenDialog;
    dlgSave: TSaveDialog;
    HueTrackBar: THueTrackBar;
    SatTrackBar: TBWTrackBar;
    ColorBoxOld: TColorBox;
    ColorBoxNew: TColorBox;
    ColorQuad: TColorQuad;
    AlphaTrackBar: TAlphaTrackBar;
    procedure fncButtonOpenClick(Sender: TObject);
    procedure fncButtonSaveClick(Sender: TObject);
    procedure fncButtonFirstClick(Sender: TObject);
    procedure fncButtonNextClick(Sender: TObject);
    procedure fncButtonApplyClick(Sender: TObject);
    procedure fncButtonApplyAllClick(Sender: TObject);
    procedure HueTrackBarChange(Sender: TObject);
    procedure SatTrackBarChange(Sender: TObject);
    procedure AlphaTrackBarChange(Sender: TObject);
    procedure fncRichEditorCaretChanged(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
    procedure UpdateTextColor(AElement: TREElement);
    procedure UpdateBackColor(AElement: TREElement);
    procedure UpdateColorBox(AElement: TREElement);
    procedure UpdateHeader;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.fmx}

//------------------------------------------------------------------------------
procedure TfrmMain.UpdateTextColor(AElement: TREElement);
var
  ColorStr: String;
  ColorVal: TAlphaColor;
  SL: TStringList;

begin
  SL := TStringList.Create;
  try
    SL.Add(AElement.Text);
    ColorStr := SL.Values[SL.Names[0]];
    ColorStr := ColorStr.Substring(1);
    ColorVal := StringToAlphaColor(ColorStr);
    AElement.Font.Style := [TFontStyle.fsBold];
    AElement.TextColor := ColorVal;
    AElement.Color := claWhite;
  finally
    SL.Free;
  end;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.UpdateBackColor(AElement: TREElement);
var
  ColorStr: String;
  ColorVal: TAlphaColor;
  SL: TStringList;

begin
  SL := TStringList.Create;
  try
    SL.Add(AElement.Text);
    ColorStr := SL.Values[SL.Names[0]];
    ColorStr := ColorStr.Substring(1);
    ColorVal := StringToAlphaColor(ColorStr);
    AElement.TextColor := claBlack;
    AElement.Color := ColorVal;
  finally
    SL.Free;
  end;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.UpdateColorBox(AElement: TREElement);
var
  H, S, L, AlphaVal: Single;
  ColorStr, AlphaStr: String;
  ColorVal: TAlphaColor;
  SL: TStringList;

begin
  SL := TStringList.Create;
  try
    SL.Add(AElement.Text);
    ColorStr := SL.Values[SL.Names[0]];
    ColorStr := ColorStr.Substring(1);
    if ColorStr.SubString(0,3) = 'cla' then
    begin
      AlphaVal := 1.0;
    end
    else
    begin
      AlphaStr := ColorStr.Substring(1,2);
      AlphaVal := StrToInt('$' + AlphaStr)/255;
    end;

    ColorVal := StringToAlphaColor(ColorStr);
    ColorBoxOld.Color := ColorVal;
    ColorBoxNew.Color := ColorVal;

    System.UIConsts.RGBtoHSL(ColorBoxOld.Color, H, S, L);

    ColorQuad.Hue := H;
    ColorQuad.Sat := S;
    ColorQuad.Lum := L;
    ColorQuad.Alpha := AlphaVal;

    HueTrackBar.Value := H;
    SatTrackBar.Value := S;
    AlphaTrackBar.Value := AlphaVal;

    if AElement.Color = StringToAlphaColor('claWhite') then
      AElement.TextColor := ColorVal
    else
      AElement.Color := ColorVal;

  finally
    SL.Free;
  end;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.UpdateHeader;
var
  Index: Integer;
  ColorStr, Header: String;
  Element: TREElement;
  SL: TStringList;

begin
  Element := fncRichEditor.Caret.Element;
  if Assigned(Element) then
  begin
    Index := fncRichEditor.Context.Content.IndexOf(Element);

    SL := TStringList.Create;
    try
      SL.Add(Element.Text);
      ColorStr := SL.Values[SL.Names[0]];
      ColorStr := ColorStr.Substring(1);
      Header := 'Index: ' + IntToStr(Index);
      Header := Header + '  ;  ';
      Header := Header + 'Color = ' + ColorStr;
      Header := Header + '  ;  ';
      Header := Header + 'Hue = ' + FloatToStrF(HueTrackBar.Value*360, ffFixed, 5, 0);
      Header := Header + '  ;  ';
      Header := Header + 'Sat = ' + FloatToStrF(SatTrackBar.Value, ffFixed, 3, 2);
      Header := Header + '  ;  ';
      Header := Header + 'Alpha = ' + FloatToStrF(AlphaTrackBar.Value, ffFixed, 3, 2);
      fncPanel.Header.Text := Header;
    finally
      SL.Free;
    end;

  end;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.fncButtonOpenClick(Sender: TObject);
var
  Element: TREElement;

begin
  fncRichEditor.Clear;
  if dlgOpen.Execute then
  begin
    fncRichEditor.LoadFromTextFile(dlgOpen.FileName);
    fncStatusBar.Panels.Items[1].Text := dlgOpen.FileName;
    fncButtonFirstClick(Self);

    fncButtonSave.Enabled := True;
    fncButtonFirst.Enabled := True;
    fncButtonNext.Enabled := True;
    fncButtonApply.Enabled := True;
    fncButtonApplyAll.Enabled := True;
    ColorQuad.Enabled := True;
    HueTrackBar.Enabled := True;
    SatTrackBar.Enabled := True;

    fncRichEditor.BeginUpdate;
    for var i := 0 to fncRichEditor.Context.Content.Count-1 do
    begin
      Element := fncRichEditor.Context.Content.Items[i];
      if ContainsText(Element.Text, '= x') or ContainsText(Element.Text, '= cla') then
        UpdateBackColor(Element)
    end;
    fncRichEditor.EndUpdate;

    UpdateHeader;

  end;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.fncButtonSaveClick(Sender: TObject);
var
  SL: TStringList;

begin
  if dlgSave.Execute then
  begin
    SL := TStringList.Create;
    try
      SL.Add(fncRichEditor.PlainText);
      SL.SaveToFile(dlgSave.FileName);
    finally
      SL.Free;
    end;
  end;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.fncButtonFirstClick(Sender: TObject);
begin
  if fncRichEditor.FindFirst('= x') then
  begin
    fncRichEditor.ScrollToCaret;
    UpdateColorBox(fncRichEditor.Caret.Element);
    UpdateHeader;
  end;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.fncButtonNextClick(Sender: TObject);
begin
  if fncRichEditor.FindNext then
  begin
    fncRichEditor.ScrollToCaret;
    UpdateColorBox(fncRichEditor.Caret.Element);
    UpdateHeader;
  end;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.fncButtonApplyClick(Sender: TObject);
var
  ColorVal: TAlphaColor;
  ColorStr: String;
  Element: TREElement;
  SL: TStringList;

begin
  Element := fncRichEditor.Caret.Element;
  if ContainsText(Element.Text, '= x') or ContainsText(Element.Text, '= cla') then
  begin
    ColorVal := ColorBoxNew.Color;
    ColorStr := AlphaColorToString(ColorVal);

    SL := TStringList.Create;
    try
      SL.Add(Element.Text);

      if ColorStr.Substring(0,1) <> '#' then
        ColorStr := 'cla' + ColorStr
      else
        Element.Text := SL.Names[0] + '= ' + ColorStr.Replace('#','x');

      UpdateBackColor(Element);
    finally
      SL.Free;
    end;

  end;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.fncButtonApplyAllClick(Sender: TObject);
var
  ColorVal, ColorOld, ColorNew: TAlphaColor;
  ColorStr: String;
  Element: TREElement;
  SL: TStringList;

begin
  for var i := 0 to fncRichEditor.Context.Content.Count-1 do
  begin
    Element := fncRichEditor.Context.Content.Items[i];
    if ContainsText(Element.Text, '= x') or ContainsText(Element.Text, '= cla') then
    begin
      ColorOld := ColorBoxOld.Color;
      ColorNew := ColorBoxNew.Color;

      SL := TStringList.Create;
      try
        SL.Add(Element.Text);
        ColorStr := SL.Values[SL.Names[0]];
        ColorStr := ColorStr.Substring(1);
        ColorVal := StringToAlphaColor(ColorStr);
        if ColorVal = ColorOld then
        begin
          SL.Add(Element.Text);
          ColorStr := AlphaColorToString(ColorNew);

          if ColorStr.Substring(0,1) <> '#' then
            ColorStr := 'cla' + ColorStr
          else
            Element.Text := SL.Names[0] + '= ' + ColorStr.Replace('#','x');

          UpdateTextColor(Element);
        end;
      finally
        SL.Free;
      end;

    end;
  end;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.HueTrackBarChange(Sender: TObject);
begin
  ColorQuad.Hue := HueTrackBar.Value;
  UpdateHeader;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.SatTrackBarChange(Sender: TObject);
begin
  ColorQuad.Sat := SatTrackBar.Value;
  UpdateHeader;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.AlphaTrackBarChange(Sender: TObject);
begin
  ColorQuad.Alpha := AlphaTrackBar.Value;
  UpdateHeader;
end;


//------------------------------------------------------------------------------
procedure TfrmMain.fncRichEditorCaretChanged(Sender: TObject);
var
  Element: TREElement;

begin
  Element := fncRichEditor.Caret.Element;
  if ContainsText(Element.Text, '= x') or ContainsText(Element.Text, '= cla') then
  begin
    UpdateColorBox(fncRichEditor.Caret.Element);
    UpdateHeader;
  end;
end;


end.
